@@ -2,14 +2,14 @@
 API Definition helpers for user apps
 """
 import inspect
-import warnings
 from functools import partial
 from typing import Optional, List, Type, Dict, Callable, Union, Tuple, Any, TypeVar
 
 import re
 import typing_inspect  # type: ignore
 
-from hopeit.app.config import AppConfig, AppDescriptor
+from hopeit.dataobjects import BinaryDownload
+from hopeit.app.config import AppConfig, AppDescriptor, EventType
 from hopeit.server.api import spec, app_route_name, APIError, BUILTIN_TYPES, datatype_schema
 from hopeit.server.names import route_name
 
@@ -65,16 +65,16 @@ def _arg_name(arg: ArgDef) -> str:
     return arg[0]
 
 
-def _arg_type(arg: ArgDef) -> Tuple[str, bool]:
+def _arg_type(arg: ArgDef) -> Tuple[str, Optional[str], bool]:
     if isinstance(arg, str):
-        return BUILTIN_TYPES[str], True
+        return (*BUILTIN_TYPES[str], True)
     datatype, required = arg[1], True
     origin = typing_inspect.get_origin(datatype)
     if origin is Union:
         type_args = typing_inspect.get_args(datatype)
         datatype = type_args[0]
         required = type_args[-1] is None
-    return BUILTIN_TYPES.get(datatype, BUILTIN_TYPES[str]), required
+    return (*BUILTIN_TYPES.get(datatype, BUILTIN_TYPES[str]), required)
 
 
 def _arg_description(arg: ArgDef) -> str:
@@ -116,26 +116,16 @@ def _method_description(module: str, description: Optional[str] = None, summary:
     return _method_summary(module, summary)
 
 
-def _event_api(
-        summary: Optional[str],
-        description: Optional[str],
-        payload: Optional[Type],
-        query_args: Optional[List[ArgDef]],
-        responses: Optional[Dict[int, PayloadDef]],
-        module, app_config: AppConfig, event_name: str, plugin: Optional[AppConfig]) -> dict:
+def _parse_args_schema(query_args: Optional[List[ArgDef]]):
     """
-    Handler returned by event_api(...)
+    Parse query args schema
     """
-    if query_args is None:
-        query_args = []
-    if responses is None:
-        responses = {}
     parameters = []
-    for query_arg in query_args:
+    for query_arg in (query_args or []):
         arg_name = _arg_name(query_arg)
-        arg_type, arg_req = _arg_type(query_arg)
+        arg_type, arg_format, arg_req = _arg_type(query_arg)
         arg_desc = _arg_description(query_arg)
-        parameters.append({
+        arg_spec: dict = {
             "name": arg_name,
             "in": "query",
             "required": arg_req,
@@ -143,43 +133,112 @@ def _event_api(
             "schema": {
                 "type": arg_type
             }
-        })
+        }
+        if arg_format is not None:
+            arg_spec["schema"]["format"] = arg_format
+        parameters.append(arg_spec)
+    return parameters
+
+
+def _parse_fields_schema(fields: Optional[List[ArgDef]]):
+    """
+    Parse form fields schema
+    """
+    fields_schema = {"type": "object", "required": [], "properties": {}}
+    encoding = {}
+    for field in (fields or []):
+        arg_name = _arg_name(field)
+        arg_type = str if len(field) == 1 else field[1]
+        arg_schema = datatype_schema(arg_name, arg_type)  # type: ignore
+        fields_schema['required'].append(arg_name)  # type: ignore
+        props = arg_schema.get('properties', {arg_name: arg_schema})
+        fields_schema['properties'].update(props)  # type: ignore
+        if props[arg_name].get('type') == 'string':
+            encoding[arg_name] = {
+                'contentType':
+                'application/octect-stream' if props[arg_name].get('format') == 'binary' else 'text/plain'
+            }
+        else:
+            encoding[arg_name] = {'contentType': 'application/json'}
+        arg_desc = _arg_description(field)
+        if arg_desc is not None:
+            fields_schema['properties'][arg_name]['description'] = arg_desc  # type: ignore
+    return fields_schema, encoding
+
+
+def _event_api(
+        summary: Optional[str],
+        description: Optional[str],
+        payload: Optional[Type],
+        query_args: Optional[List[ArgDef]],
+        fields: Optional[List[ArgDef]],
+        responses: Optional[Dict[int, PayloadDef]],
+        module, app_config: AppConfig, event_name: str, plugin: Optional[AppConfig]) -> dict:
+    """
+    Handler returned by event_api(...)
+    """
+    parameters = _parse_args_schema(query_args)
+
     method_spec: Dict[str, Any] = {
         "summary": _method_summary(module, summary),
         "description": _method_description(module, description, summary),
         "parameters": parameters
     }
+
+    event_config = app_config.events[event_name]
+    content_type = 'multipart/form-data' if event_config.type == EventType.MULTIPART else 'application/json'
     if payload is not None:
         method_spec['requestBody'] = {
             "description": _payload_description(payload),
             "required": True,
             "content": {
-                "application/json": {
+                content_type: {
                     "schema": _payload_schema(event_name, payload)
                 }
             }
         }
 
+    def _payload_content_type(arg: PayloadDef) -> Tuple[PayloadDef, str]:
+        dt = arg[0] if isinstance(arg, tuple) else arg
+        if inspect.isclass(dt) and issubclass(dt, BinaryDownload):
+            return (BinaryDownload, arg[1]) if isinstance(arg, tuple) else BinaryDownload, dt.content_type
+        return arg, "application/json"
+
+    def _response_content(datatype) -> dict:
+        dt, content_type = _payload_content_type(datatype)
+        return {content_type: {
+                "schema": _payload_schema(event_name, dt)}}
+
+    if fields is not None:
+        if payload is not None:
+            raise APIError("Payload and fields cannot be specified at the same time.")
+
+        method_schema, encoding = _parse_fields_schema(fields)
+        method_spec['requestBody'] = {
+            "required": True,
+            "content": {
+                content_type: {
+                    "schema": method_schema,
+                    "encoding": encoding
+                }
+            }
+        }
+
     api_responses = {
         str(status): {
             "description": _payload_description(datatype),
-            "content": {
-                "application/json": {
-                    "schema": _payload_schema(event_name, datatype)
-                }
-            }
-        } for status, datatype in responses.items()}
+            "content": _response_content(datatype)
+        } for status, datatype in (responses or {}).items()}
     method_spec['responses'] = api_responses
     return method_spec
 
 
-def event_api(title: Optional[str] = None,
+def event_api(*, summary: Optional[str] = None,
+              description: Optional[str] = None,
               payload: Optional[PayloadDef] = None,
               query_args: Optional[List[ArgDef]] = None,
-              responses: Optional[Dict[int, PayloadDef]] = None, *,
-              summary: Optional[str] = None,
-              description: Optional[str] = None
-              ) -> Callable[..., dict]:
+              fields: Optional[List[ArgDef]] = None,
+              responses: Optional[Dict[int, PayloadDef]] = None) -> Callable[..., dict]:
     """
     Provides a convenient way to define Open API specification using Python types for a given app event
     implementation module.
@@ -192,8 +251,6 @@ def event_api(title: Optional[str] = None,
         float, bool), or a tuple of (str, type, str) where last string is argument description.
     :param responses: a dictionary where key HTTP status code and value is the payload definition as describer in
         payload parameter.
-    :param title: Deprecated, use summary instead.
-
 
     Examples:
 
@@ -213,11 +270,4 @@ def event_api(title: Optional[str] = None,
         )
     """
 
-    if title is not None:
-        warnings.warn(
-            "title parameter is deprecated since 0.1.4 and will be removed in version 0.2.0, use summary instead",
-            DeprecationWarning)
-        if summary is None:
-            summary = title
-
-    return partial(_event_api, summary, description, payload, query_args, responses)
+    return partial(_event_api, summary, description, payload, query_args, fields, responses)
